@@ -55,18 +55,18 @@ class BBOXPoint():
         return cls(**data)
 
 class BBOXNormalizedLine():
-    def __init__(self, Idx, BoundingBox: List[BBOXPoint], Text:str = None, merged:bool=False, avg_height=0.0, std_height=0.0):
-        self.start_idx=Idx
-        self.end_idx=Idx
+    def __init__(self, Idx, BoundingBox: List[BBOXPoint], Text:str = None, merged:bool=False, avg_height=0.0, std_height=0.0, words_count=0):
+        self.start_idx=Idx+1
+        self.end_idx=Idx+1
         self.boundingbox = BoundingBox
-        self.text = ''
+        self.text = None
+        self.words_count=words_count
         self.merged = merged
         self.calculateMedians()
         self.__appendText(Text)
         self.avg_height = avg_height
         self.std_height = std_height
         self.rank=0.0
-        self.listids=[]
 
     def calculateMedians(self):
         self.xmedian=(min(self.boundingbox[0].X,self.boundingbox[3].X) + max(self.boundingbox[1].X,self.boundingbox[2].X))/2
@@ -81,11 +81,28 @@ class BBOXNormalizedLine():
             result.append([box.X*scale,box.Y*scale])
         return result
 
+    def getWidthRange(self,scale=1):
+        return (int(self.boundingbox[0].X*scale),int(self.boundingbox[1].X*scale))
+
+    def getHeightRange(self,scale=1):
+        return (int(self.boundingbox[0].Y*scale),int(self.boundingbox[3].Y*scale))
+
     def getBoxesAsRectangle(self,scale=1):
-        return (self.boundingbox[0].X*scale,self.boundingbox[0].Y*scale,(self.boundingbox[2].X-self.boundingbox[0].X)*scale,(self.boundingbox[2].Y-self.boundingbox[0].Y)*scale)
+        return (int(self.boundingbox[0].X*scale),int(self.boundingbox[0].Y*scale),int(self.boundingbox[2].X-self.boundingbox[0].X)*scale,int(self.boundingbox[2].Y-self.boundingbox[0].Y)*scale)
 
     def __appendText(self, Text):
-        self.text += Text
+        # TODO #2
+        if self.text:
+            # Word ceasure with a single dash
+            if self.text.endswith('-'):
+                self.text = self.text[:-1]+Text.strip()
+            else:
+                self.text += (" " + Text.strip())
+                pass
+        else:
+            self.text=Text.strip()
+
+        # Flag for sentence for future use (NLP support)
         if Text.endswith('.'):
             self.end_sentence=True
         else:
@@ -93,13 +110,14 @@ class BBOXNormalizedLine():
 
     def appendLine(self,line):
         line.merged = True
-        self.__appendText(" " + line.text)
+        self.__appendText(line.text)
         self.boundingbox[0] = BBoxUtils.minXminY(0,self,line)
         self.boundingbox[1] = BBoxUtils.maxXminY(1,self,line)
         self.boundingbox[2] = BBoxUtils.maxXmaxY(2,self,line)
         self.boundingbox[3] = BBoxUtils.minXmaxY(3,self,line)
         self.calculateMedians()
         self.end_idx=line.end_idx
+        self.words_count+=line.words_count
         
     @classmethod
     def from_azure(cls, index, data, ppi):
@@ -121,13 +139,13 @@ class BBOXNormalizedLine():
 
             # Make sure the X of the line is consistent to the first word of the same. 
             if len(data['words'])>0:
-                points[0].X = data['words'][0]["boundingBox"][0]+1
-                points[3].X = data['words'][0]["boundingBox"][6]+1
+                points[0].X = max(data['words'][0]["boundingBox"][0],points[0].X)
+                points[3].X = max(data['words'][0]["boundingBox"][6],points[3].X)
         else:
             points = list(map(BBOXPoint.from_azure, [ppi*x for x in array]))
             if len(data['words'])>0:
-                points[0].X = data['words'][0]["boundingBox"][0].X+1
-                points[3].X = data['words'][0]["boundingBox"][3].X+1
+                points[0].X = max(data['words'][0]["boundingBox"][0].X,points[0].X)
+                points[3].X = max(data['words'][0]["boundingBox"][3].X,points[3].X)
           
         wordheights=[]
         # # Check Line in anomaly
@@ -138,10 +156,10 @@ class BBOXNormalizedLine():
         npheights = np.array(wordheights)
         avg_height = np.average(npheights)
         std_height = np.std(npheights, dtype=np.float64)
-        return cls(Idx=index,BoundingBox=points,Text=data['text'],std_height=std_height,avg_height=avg_height)
+        return cls(Idx=index,BoundingBox=points,Text=data['text'],std_height=std_height,avg_height=avg_height,words_count=len(data['words']))
 
     @classmethod
-    def from_google(cls, line_counter, line_text, line_boxes):
+    def from_google(cls, line_counter, line_text, line_boxes, words_count):
         points = list()
         for bb in range(4):
             points.append(BBOXPoint(0,0))
@@ -156,7 +174,7 @@ class BBOXNormalizedLine():
         points[2].X = line_boxes[-1][2].x 
         points[2].Y = line_boxes[-1][2].y 
 
-        return BBOXNormalizedLine(Idx=line_counter,BoundingBox=points,Text=line_text)
+        return BBOXNormalizedLine(Idx=line_counter,BoundingBox=points,Text=line_text,words_count=words_count)
 
 class BBOXPageLayout():
     def __init__(self, Id:int = 0,clockwiseorientation:float = 0.0,Width:float = 0.0,Height:float = 0.0,Unit:str = "pixel",Language:str="en",Text:str=None,Lines:List[BBOXNormalizedLine]=None,ppi=1):
@@ -196,10 +214,13 @@ class BBOXPageLayout():
 
     @classmethod
     def from_google(cls, page):
+        # Initialization
         lines=[]
         line_counter=0
         line_text =""
         line_boxes=[]
+        line_words_count=0
+
         bboxlogger.debug("Google|Page shape (Height,Width) ({0},{1})".format(page.height,page.width))
         # Create the concept of lines for Google ocr response. 
         for idb, block in enumerate(page.blocks):
@@ -238,14 +259,16 @@ class BBOXPageLayout():
                             # if xdiff > bboxconfig.config["pixel"].GoogleLineBreakThresholdInPixel:
                                 bboxlogger.debug("Google|Detected Cluster Break current {0} found {1} | {2} {3}".format(currentTextColumn,foundTextColumn,str(line_counter),line_text))
                                 # Line break
-                                line=BBOXNormalizedLine.from_google(line_counter,line_text,line_boxes)
+                                line=BBOXNormalizedLine.from_google(line_counter,line_text,line_boxes,words_count=line_words_count)
                                 lines.append(line)
                                 line_text=""
                                 line_counter+=1
                                 line_boxes.clear()
-                    currentTextColumn=foundTextColumn
+                                line_words_count=0
 
-                    line_boxes.append(word.bounding_box.vertices)
+                    currentTextColumn=foundTextColumn
+                    line_boxes.append(word.bounding_box.vertices)                   
+                    line_words_count+=1
                     for symbol in word.symbols:
                         line_text+=symbol.text
                         if symbol.property.detected_break:
@@ -254,11 +277,12 @@ class BBOXPageLayout():
                             elif symbol.property.detected_break.type in [3,5]:
                                 bboxlogger.debug("Google|Detected Line Break {0}| {1} {2}".format(str(symbol.property.detected_break.type),str(line_counter),line_text))
                                 # Line Break
-                                line=BBOXNormalizedLine.from_google(line_counter,line_text,line_boxes)
+                                line=BBOXNormalizedLine.from_google(line_counter,line_text,line_boxes,words_count=line_words_count)
                                 lines.append(line)
                                 line_text=""
                                 line_counter+=1
                                 line_boxes.clear()
+                                line_words_count=0                                
 
         return cls(Id=1,Width=page.width,Height=page.height,Lines=lines)
 
@@ -409,8 +433,9 @@ class BBoxHelper():
             if self.annotate and bboxannotate.pageTag:
                 newtext+=bboxannotate.pageTag[1]
             else:
+                # TODO #3
                 # Default page separator
-                newtext += '\r\n'
+                newtext += os.linesep
 
             # Revert the rotation of the bounding boxes back to its original orientation
             if applied_rotation!=0:
@@ -494,9 +519,9 @@ class BBoxHelper():
                 # //Top Left Y
                 ycurrent = line.boundingbox[boxref].Y
                 # lowb=(regiony - (Ythresholdratio * bboxconfig.config[unit].ImageTextBoxingYThreshold))
-                lowb=regiony-(regiony*0.1)
+                lowb=regiony-(regiony*0.05)
                 highb=(regiony + (Ythresholdratio * bboxconfig.get_ImageTextBoxingYThreshold(unit,ppi)))
-                bboxlogger.debug("{7}|Line bbox {0} {6} {1} | LowY {2}<{3}<{4} HighY | Merge {5}".format(str(line.boundingbox),str(line.text),str(lowb),str(ycurrent),str(highb),str((ycurrent >= lowb and ycurrent < highb)),str(regiony),alignment))
+                bboxlogger.debug("{7}|Line bbox {0} {6} {1} | LowY {2}<{3}<{4} HighY | Merge {5}".format(str(line.boundingbox),str(line.text),str(lowb),str(ycurrent),str(highb),str((ycurrent >= lowb and ycurrent <= highb)),str(regiony),alignment))
 
                 if (regiony == 0.0):
                     prevline = line
@@ -527,6 +552,10 @@ class BBoxHelper():
             bboxlogger.debug("{1}|Processing {0}".format(alignment,str(page.id)))
             page.lines = self.__processLineBoundingBoxes(page.lines,alignment,page.unit,page.ppi)
 
+        # Another pass on LeftAlignment
+        bboxlogger.debug("{1}-2|Processing {0}".format(alignment,str(page.id)))
+        page.lines = self.__processLineBoundingBoxes(page.lines,LeftAlignment,page.unit,page.ppi)
+
         outlines=[o for o in page.lines if o.merged == False]
         bboxlogger.debug("{1}|Output {0} lines before sorting...".format(len(outlines),str(page.id)))
         #
@@ -574,13 +603,13 @@ class BBoxHelper():
                         # if block.getClusterId() != sortedBlocks[i+1].getClusterId() and (not sortedBlocks[i+1].text[0].isupper()):
                         if not sortedBlocks[i+1].text[0].isupper():
                             if block.text.strip().endswith("."):
-                                newtext+='\r\n'
+                                newtext+=os.linesep
                             else:
                                 newtext+=' '
                         # elif (not sortedBlocks[i+1].text[0].isupper()):
                         #     newtext+=' '
                         else:
-                            newtext+='\r\n'
+                            newtext+=os.linesep
                     # Default separator
                     # if block.Text.strip().endswith("."):
                     # newtext+='\r\n'
