@@ -12,10 +12,6 @@ import uuid
 from datetime import datetime
 from os import path
 from typing import List
-import cv2
-import numpy as np
-from PIL import Image, ImageDraw
-import cv2
 import numpy as np
 
 from . import bboxlog
@@ -133,13 +129,13 @@ class BBoxUtils():
         line.boundingbox[2].Y=line.boundingbox[3].Y = max(line.boundingbox[2].Y,line.boundingbox[3].Y)
 
     @classmethod
-    def draw_boxes_on_page(cls,image,blocks,color,scale=1,padding=0):
-        """Draw the blocks of text in an image using OpenCV."""
+    def draw_boxes_on_page(cls,canvas,blocks,color,scale=1,padding=0):
+        """Draw the blocks of text in a vanilla canvas."""
         for block in blocks:
-            pts = np.array(block.getBoxesAsArray(scale), np.int32)
-            pts = pts.reshape((-1,1,2))
-            cv2.polylines(image,[pts],True,(255,255,255))
-        return image
+            (minh,maxh)=block.getHeightRange(scale)
+            (minw,maxw)=block.getWidthRange(scale)
+            canvas[minh:maxh,minw:maxw]=block.start_idx
+        return canvas
 
     # @classmethod
     # def determine_ppi(cls,width,height):
@@ -187,14 +183,18 @@ class BBoxSort():
 
     @classmethod
     def contoursSort(cls,pageId,width,height,blocks,scale=1):
+        """Main sorting method to approach the natural reading."""
         bboxlogger.debug("{0}|contoursSort width:{1} height:{2} scale:{3}".format(str(pageId),str(width),str(height),str(scale)))
-        # Make empty black image
-        image=np.zeros((int(height*scale),int(width*scale)),np.uint8)
-        img = BBoxUtils.draw_boxes_on_page(image,blocks,"white",scale)
-        # Cluster the blocks by Y 
-        lineContours=cls.__clusterBlocks(img,blocks,scale)
+        # Make empty canvas
+        canvas=np.zeros((int(height*scale),int(width*scale)),np.uint8)
+        # Draw all the boxes on the canvas
+        canvas=BBoxUtils.draw_boxes_on_page(canvas,blocks,"white",scale)
+        # Cluster the blocks 
+        lineContours=cls.__clusterBlocks(canvas,blocks,width,height,scale)
         # Sort list on block rank
-        contours_sorted = sorted(lineContours,key= lambda o: o.rank)
+        # contours_sorted = sorted(lineContours,key= lambda o: o.rank)
+        contours_sorted = sorted(lineContours,key= lambda o: o.sorting)
+
         return contours_sorted
 
     @classmethod
@@ -215,74 +215,133 @@ class BBoxSort():
                 # when the value changed to a 0, the previous rows
                 # contained contours, so add start/end index to list
                 if val == 0:
-                    if gaplength >= gapthreshhold:
-                        clusters.append((startindex,i,gaplength))
+                    if gaplength > gapthreshhold:
+                        clusters.append((startindex,i-1,gaplength))
                     else:
                         if len(clusters)>0:
                             (start,end,gap)=clusters[-1]
-                            clusters[-1]=(start,i,gap)
+                            clusters[-1]=(start,i-1,gap)
                         else:
-                            clusters.append((startindex,i,gaplength))
-                    startindex = i+1
+                            clusters.append((startindex,i-1,gaplength))
+                    # startindex = i+1
+                    startindex = 0
                     gaplength=1
                 compVal = not compVal
             else:
                 if val==0:
                     gaplength+=1
-            if i == len(sumAxis):
-                if len(clusters)>0:
-                    (start,end,gap)=clusters[-1]
-                    clusters[-1]=(start,i,gap)
-                else:
+
+            if i == (len(sumAxis)-1):
+                if startindex>0:
                     clusters.append((startindex,i,gaplength))
+                # if len(clusters)>0:
+                #     (start,end,gap)=clusters[-1]
+                #     clusters[-1]=(start,i,gap)
+                # else:
+                #     clusters.append((startindex,i,gaplength))
 
         return clusters
 
     @classmethod
-    def __clusterBlocks(cls,img,blocks,scale=1,axis=0):
+    def __clusterBlocks(cls,canvas,blocks,width,height,scale=1,axis=0):
         # try to identify clusters of blocks
-        clusters=cls.findClusters(img,axis)
+        clusters=cls.findClusters(canvas,axis)
 
         # if there is only single cluster then we shall revert to the opposite axis strategy
         if len(clusters)<=1:
             axis=np.absolute(axis-1)
-            clusters=cls.findClusters(img,axis)
+            clusters=cls.findClusters(canvas,axis)
 
         bboxlogger.debug("Found {0} clusters on axis {1}".format(str(len(clusters)),str(axis)))
 
+        # Width*Height
+        wh=width*height
+
         lineContours = []
         # loop contours, find the boundingrect,
         # compare to line-values
         # store line number,  x value and contour index in list
         for i,cluster in enumerate(clusters):
+            lines_counter=0
+            (startindex,endindex,gap) = cluster
+            bboxlogger.debug("Cluster {0} StartIndex {1} EndIndex {2} Gap {3}".format(i,startindex,endindex,gap))
+            # Identify if we could find subclusters i.e. handling multi columns layout
+            if axis==1:
+                subclusters=cls.findClusters(canvas[cluster[0]:cluster[1],:],axis=0)
+            else:
+                subclusters=cls.findClusters(canvas[:,cluster[0]:cluster[1]],axis=1)
+
+            bboxlogger.debug("Cluster {0} - Found {1} Axis {2} sub-cluster(s)".format(i,len(subclusters),np.absolute(axis-1)))
+                
             for j,block in enumerate(blocks):
                 (x,y,w,h) = block.getBoxesAsRectangle(scale)
                 if axis==1:
-                    # Horizontal
-                    if y >= cluster[0] and y <= cluster[1]:
-                        # lineContours.append([line[0],x,j])
-                        block.rank+=i
-                        block.listids.append(block.rank)
-                        # block.rank+=(block.xmedian/(np.shape(img)[1]/scale))
-                        block.rank+=(x/np.shape(img)[1])
-                        block.listids.append(block.rank)
-                        #
-                        block.rank+=(block.ymedian/(np.shape(img)[0]/scale))*0.01
-                        block.listids.append(block.rank)
+                    # Y-axis clusters
+                    if y >= startindex and y <= endindex:
+                        lines_counter+=1
+
+                        block.cluster=i
+                        block.subcluster=0
+                        if len(subclusters)>1:
+                            # Do something special
+                            bboxlogger.debug("Sub-Cluster X-Axis processing...")
+                            for ic,subc in enumerate(subclusters):
+                                (subcstart,subcend,subcgap)=subc
+                                if x >= subcstart and x <= subcend:
+                                    block.subcluster=ic
+                                    break
+                            block.sorting=(block.cluster,block.subcluster,block.ymedian)
+                        else:
+                            block.rank+=(block.xmedian/(np.shape(canvas)[1]/scale))
+                            block.rank+=(block.ymedian/(np.shape(canvas)[0]/scale))*0.01
+                            block.sorting=(block.cluster,block.subcluster,block.rank)
+                            pass
+                        # block.rank+=(block.xmedian*block.ymedian)/scale/wh
+                        # block.listids.append(block.rank)
+
+                        # # block.rank+=(block.xmedian/(np.shape(img)[1]/scale))
+                        # block.rank+=(x/np.shape(img)[1])
+                        # block.listids.append(block.rank)
+                        # #
+                        # block.rank+=(block.ymedian/(np.shape(img)[0]/scale))*0.01
+                        # block.listids.append(block.rank)
+
                         lineContours.append(block)
                 else:
-                    # Vertical
-                    if x >= cluster[0] and x <= cluster[1]:
-                        # the cluster id weight
-                        block.rank+=i
-                        block.listids.append(block.rank)
-                        # the vertical median weight
-                        # block.rank+=(block.ymedian/(np.shape(img)[0]/scale))
-                        block.rank+=(y/np.shape(img)[0])
-                        block.listids.append(block.rank)
-                        # the horizontal median weight
-                        block.rank+=(block.xmedian/(np.shape(img)[1]/scale))*0.01
-                        block.listids.append(block.rank)
+                    # X-axis clusters
+                    if x >= startindex and x <= endindex:
+                        lines_counter+=1
+                        # # the cluster id weight
+                        # block.rank+=i
+                        # block.listids.append(block.rank)
+                        # # the vertical median weight
+                        # # block.rank+=(block.ymedian/(np.shape(img)[0]/scale))
+                        # block.rank+=(y/np.shape(canvas)[0])
+                        # block.listids.append(block.rank)
+                        # # the horizontal median weight
+                        # block.rank+=(block.xmedian/(np.shape(canvas)[1]/scale))*0.01
+                        # block.listids.append(block.rank)
+
+                        block.cluster=i
+                        block.subcluster=0
+                        if len(subclusters)>1:
+                            # Do something special
+                            bboxlogger.debug("Sub-Cluster Y-Axis processing...")
+                            for ic,subc in enumerate(subclusters):
+                                (subcstart,subcend,subcgap)=subc
+                                if y >= subcstart and y <= subcend:
+                                    block.subcluster=ic
+                                    break
+                            block.sorting=(block.cluster,block.subcluster,block.xmedian)
+                        else:
+                            block.rank+=(block.ymedian/(np.shape(canvas)[0]/scale))
+                            block.rank+=(block.xmedian/(np.shape(canvas)[1]/scale))*0.01
+
+                            block.sorting=(block.cluster,block.subcluster,block.rank)
+                            pass
+
                         lineContours.append(block)
 
-        return lineContours
\ No newline at end of file
+            bboxlogger.debug("Cluster {0} assigned to {1} lines".format(i,lines_counter))
+
+        return lineContours
